# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

def powerset(x):
    """ Generator which yields all sublists of the given list (including itself and the empty list)
    """
    if len(x)==0: 
        yield []
        return
    subp = powerset(x[1:])
    for s in subp:
        yield [x[0]]+s
        yield s

def permutations(x):
    """ Generator which yields all permutations of the given list
    """
    if len(x)<=1: 
        yield x
        return
    for sp in permutations(x[1:]):
        for i in xrange(len(x)):
            yield sp[:i] + [x[0]] + sp[i:]

def first_item(items, defvalue=None):
    """ Returns the first item from the given iterable, or defvalue (defaults to None) if the iterable has no items
    """
    for i in items:
        return i
    return defvalue
    
def convert_collections(obj, collection_type):
    """ Converts any lists/tuples in obj (assuming it's a nested dict/list/tuple structure) to collection_type """
    if isinstance(obj, (list,tuple)):
        return collection_type(convert_collections(o, collection_type) for o in obj)
    elif isinstance(obj, dict):
        return dict(((k,convert_collections(v, collection_type)) for k,v in obj.iteritems()))
    else:
        return obj
        
def lists_to_tuples(obj):
    return convert_collections(obj, tuple)
    
def tuples_to_lists(obj):
    return convert_collections(obj, list)
    
def open_url_from_path(path):
    import urllib, webbrowser, os, os.path
    fileurl = urllib.pathname2url(path)
    # if not absolute path, prepend current directory
    if not fileurl.startswith('/'):
        fileurl = urllib.pathname2url(os.path.join(os.getcwd(), path))
    # Windows adds leading '//' in pathname2url, Mac and Linux don't
    if not fileurl.startswith('//'):
        fileurl = '//' + fileurl
    webbrowser.open_new('file:'+fileurl)
    
def import_json():
    """ Looks for JSON module in json (Python 2.6) or simplejson, returns object supporting loads/dumps
        via eval/repr if neither is found. """
    try:
        return __import__('json')
    except:
        try:
            return __import__('simplejson')
        except:
            # create fake json module with dumps and loads functions
            print('WARNING: JSON package not found, using unsafe repr/eval')
            class dummy(object): pass
            fake_json = dummy()
            # convert tuples to lists and use '"' so repr output is json-compatible
            fake_json.dumps = lambda x: repr(tuples_to_lists(x)).replace('\'', '"')
            fake_json.loads = eval
            fake_json.__version__ = '0.1'
            fake_json.__name__ = 'fake_json'
            return fake_json

def check_app_update(url, current_version, update_callback, no_update_callback=None, error_callback=None, threaded=True):
    """ Checks for app update by retrieving version information from url. If an update is available, calls
        the update_callback function passing the version string and download page URL as arguments. If an
        error occurs and error_callback is provided, calls error_callback passing the exception object.
        If no update is available and no_update_callback is provided, it is called with no arguments.
        If threaded is true (the default), URL retrieval and callbacks run in a separate thread.
        The data at the URL is expected to be a JSON object with the following structure:
            "version_number": numeric version number, compared to current_version parameter to determine if newer
            "version_string": human-readable version
            "url": URL of the page to download update
    """
    def check():
        import urllib2
        json = import_json()
        try:
            version_dict = json.loads(urllib2.urlopen(url).read())
            if version_dict['version_number'] > current_version:
                update_callback(version_dict['version_string'], version_dict['url'])
            else:
                if no_update_callback: no_update_callback()
        except Exception, ex:
            if error_callback: 
                error_callback(ex)
    
    if threaded:
        import threading
        t = threading.Thread(target=check)
        t.setDaemon(True)
        t.start()
    else:
        check()
        